function max_geneset=mod_CoMDP(A,S,k,lambda,eta,exclusion)
%
% This function uses the MATLAB interface of CPLEX to solve the binary linear programming model for the discovery of co-occurring mutated driver pathway with a known pathway (mod_CoMDP). 
% Before running this funciton, please make sure you have installed CPLEX software in your computer.
%
% A : m (sample) x n (genes) mutation matrix
%
% S: a known pathway (gene set)
%
% k : number of desired genes 
%
% lambda : a parameter to control the common coverage of the two gene sets
%
% eta : a parameter to control the non-shared coverage of the two gene sets
%
% exclusion : the genes which are to be excluded from the results
%
% max_geneset : output vector with s_length+k+11 column (s_length equals the number of genes in S);
%               it records the information of the optimal solution; 
%               max_geneset(1:s_length) records the known gene set;
%               max_geneset(s_length+1:s_length+k) records the identified gene set;
%               max_geneset(s_length+k+1) records the weight of the gene sets;
%               max_geneset(s_length+k+2) records the significance level of the known gene set;
%               max_geneset(s_length+k+3) records the significance level of the identified gene set;
%               max_geneset(s_length+k+4) records the co-occurrence significance level of the two gene sets;
%               max_geneset(s_length+k+5) records the number of genes in the known gene set;
%               max_geneset(s_length+k+6) records the number of genes in the identified gene set;
%               max_geneset(s_length+k+7) records the coverage of the known gene set;
%               max_geneset(s_length+k+8) records the coverage of the identified gene set;
%               max_geneset(s_length+k+9) records the common coverage of the two gene sets;
%               max_geneset(s_length+k+10) records the union coverage of the two gene sets;
%               max_geneset(s_length+k+11) records the co-occurrence ratio of the two gene sets.
%


if ~isempty(exclusion)
    A(:,exclusion)=0;
end
[m n]=size(A);

s_length=length(S);
s_x=zeros(m,1);
s_y=zeros(1,n);
for i=1:m
    tem=0;
    for j=1:s_length
        if A(i,S(j))~=0
            tem=tem+1;
            if tem~=0
                s_x(i)=1;
                break;
            end
        end
    end
end
for j=1:s_length
    s_y(S(j))=1;
end

f=zeros(1,2*m+n);   % Vector containing the coefficients of the linear objective function
f(1:m)=-(eta+1);
f(m+1:m+n)=sum(A);
f(m+n+1:2*m+n)=2*eta-lambda;
B=zeros(4*m+n,2*m+n);   % Matrix containing the coefficients of the linear inequality constraints
B(1:m,1:m)=diag(ones(1,m));
B(1:m,m+1:m+n)=-A;
B(m+1:2*m,1:m)=-n.*diag(ones(1,m));
B(m+1:2*m,m+1:m+n)=A;
B(2*m+1:3*m,1:m)=-diag(ones(1,m));
B(2*m+1:3*m,m+n+1:2*m+n)=2.*diag(ones(1,m));
B(3*m+1:4*m,1:m)=diag(ones(1,m));
B(3*m+1:4*m,m+n+1:2*m+n)=-2.*diag(ones(1,m));
B(4*m+1:4*m+n,m+1:m+n)=diag(ones(1,n));
b=zeros(4*m+n,1);     % Vector corresponding to the right-hand side of the linear inequality constraints
b(2*m+1:3*m)=s_x;
b(3*m+1:4*m)=1.-s_x;
b(4*m+1:4*m+n)=1.-s_y;
Aeq=zeros(1,2*m+n); % Matrix containing the coefficients of the linear equality constraints
Aeq(m+1:m+n)=1;
beq=k;            % Vector containing the constants of the linear equality constraints

[X maxweight]= cplexbilp(f,B,b,Aeq,beq);

max_geneset(1:s_length)=S;
k=sum(X(m+1:m+n)>0.5);
max_geneset(s_length+1:s_length+k)=find(X(m+1:m+n)>0.5);
max_geneset(s_length+k+1)=-maxweight;

max_geneset(s_length+k+2)=significance_A(A,max_geneset(1:s_length));

max_geneset(s_length+k+3)=significance_A(A,max_geneset(s_length+1:s_length+k));

max_geneset(s_length+k+4)=significance_pair(A,max_geneset(1:s_length),max_geneset(s_length+1:s_length+k));
max_geneset(s_length+k+5)=s_length;
max_geneset(s_length+k+6)=k;
max_geneset(s_length+k+7:s_length+k+11)=fit_pair(A,max_geneset(1:s_length),max_geneset(s_length+1:s_length+k));


function p=significance_A(A,subset)
[m,~]=size(A);
w=zeros(1,1000);
n=length(subset);
for j=1:1000
    A_temp=A;
    A_temp(:,subset)=0;
    for i=1:n
        temp=sum(A(:,subset(i)));
        index = randperm(m,temp);
        A_temp(index,subset(i))=1;
    end
    w(j)=fit_A(A_temp,subset);
end
p=sum(w>=fit_A(A,subset))/1000;


function q=significance_pair(A,subset1,subset2)
[m,~]=size(A);
v=zeros(1,1000);
subset=[subset1 subset2];
n=length(subset);
for j=1:1000
    A_temp=A;
    A_temp(:,subset)=0;
    for i=1:n
        temp=sum(A(:,subset(i)));
        index = randperm(m,temp);
        A_temp(index,subset(i))=1;
    end
    e=fit_pair(A_temp,subset1,subset2);
    v(j)=e(5);
    clear e;
end
h=fit_pair(A,subset1,subset2);
u=h(5);
clear h;
q=sum(v>=u)/1000;


function f=fit_A(A,x)        % fitness calculation function
[~,n]=size(A);
temp=zeros(1,n);
temp(x)=1;
x=temp;
index = x==1;
A_index=A(:,index);
A_index_sum=sum(A_index,2);
f=(2*sum(A_index_sum>0)-sum(A_index_sum));


function g=fit_pair(A,x,y)
[~,n]=size(A);
temp1=zeros(1,n);
temp2=zeros(1,n);
temp1(x)=1;
temp2(y)=1;
x=temp1;
y=temp2;
index1 = x==1;
index2 = y==1;
A_index1=A(:,index1);
A_index2=A(:,index2);
A_index1_sum=sum(A_index1,2);
A_index2_sum=sum(A_index2,2);
intersec = (A_index1_sum>0)&(A_index2_sum>0);
union = (A_index1_sum>0)|(A_index2_sum>0);
g(1)=sum(A_index1_sum>0);
g(2)=sum(A_index2_sum>0);
g(3)=sum(intersec);
g(4)=sum(union);
g(5)=sum(intersec)/sum(union);
